#if !defined(RT_STATES_H)
#define RT_STATES_H
//------------------------------------------------------------
// states.h
//
// Helper functions for setting render and texture stage states.
//
#include <d3d8.h>

namespace rt
{

//------------------------------------------------------------
// s_enum_value<Enum>
// s_rs
// s_tss
//
// struct for storing a device state and its value
template <typename Enum>
struct s_enum_value
{
    Enum m_state;
    DWORD m_value;
};
typedef s_enum_value<D3DRENDERSTATETYPE> s_rs;
typedef s_enum_value<D3DTEXTURESTAGESTATETYPE> s_tss;

//------------------------------------------------------------
// set_states
//
// Set render states on the device.
//
inline void
set_states(IDirect3DDevice8 *device,
           const s_rs *states, UINT num_states)
{
    for (UINT i = 0; i < num_states; i++)
    {
        THR(device->SetRenderState(states[i].m_state,
            states[i].m_value));
    }
}

//------------------------------------------------------------
// set_states
//
// Set texture stage states on the device.
//
inline void
set_states(IDirect3DDevice8 *device, UINT stage,
           const s_tss *states, UINT num_states)
{
    for (UINT i = 0; i < num_states; i++)
    {
        THR(device->SetTextureStageState(stage,
            states[i].m_state, states[i].m_value));
    }
}

//------------------------------------------------------------
// float_dword, dword_float
//
// Reinterpret a float as a DWORD and vice-versa for device
// states represented as a float crammed into a DWORD.
//
inline DWORD
float_dword(float value)
{
    return *reinterpret_cast<DWORD *>(&value);
}
inline float
dword_float(DWORD value)
{
    return *reinterpret_cast<float *>(&value);
}

};  // rt

#endif
